package ch.hslu.appe.config;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Properties;

/**
 * Config.
 */
public final class Config {

    private static Config instance;

    /**
     * getInstance.
     * @return instance
     */
    public static Config getInstance() {
        if (instance == null) {
            instance = new Config();
        }

        return instance;
    }

    private String accessServiceWSDLUrl;
    private String userManagementServiceWSDLUrl;
    private String orderServiceWSDLUrl;
    private String inventoryServiceWSDLUrl;

    /**
     * Private Constructor.
     */
    private Config() {

        // Load config
        File configFile = new File("config.properties");
        if (configFile.exists() && !configFile.isDirectory()) {
            load();
        } else {
            // Add default values
            accessServiceWSDLUrl = "http://localhost:8080/Access?wsdl";
            userManagementServiceWSDLUrl = "http://localhost:8080/UserManagement?wsdl";
            orderServiceWSDLUrl = "http://localhost:8080/Order?wsdl";
            inventoryServiceWSDLUrl = "http://localhost:8080/Inventory?wsdl";

            // Save config
            save();
        }

    }

    /**
     * getAccessServiceWSDLUrl.
     * @return accessServiceWSDLUrl
     */
    public String getAccessServiceWSDLUrl() {
        return accessServiceWSDLUrl;
    }

    /**
     * getInventoryServiceWSDLUrl.
     * @return inventoryServiceWSDLUrl
     */
    public String getInventoryServiceWSDLUrl() {
        return inventoryServiceWSDLUrl;
    }

    /**
     * getOrderServiceWSDLUrl.
     * @return ServiceWDSLUrl
     */
    public String getOrderServiceWSDLUrl() {
        return orderServiceWSDLUrl;
    }

    /**
     * getUserManagementServiceWSDLUrl.
     * @return userManagementServiceWSDLUrl
     */
    public String getUserManagementServiceWSDLUrl() {
        return userManagementServiceWSDLUrl;
    }

    /**
     * Load.
     * @return if load ok
     */
    public boolean load() {
        Properties prop = new Properties();
        InputStream input = null;

        try {

            input = new FileInputStream("config.properties");

            // Load a properties file
            prop.load(input);

            // Get the property value and print it out
            accessServiceWSDLUrl = prop.getProperty("access-service-wsdl-url");
            userManagementServiceWSDLUrl = prop.getProperty("usermanagement-service-wsdl-url");
            orderServiceWSDLUrl = prop.getProperty("order-service-wsdl-url");
            inventoryServiceWSDLUrl = prop.getProperty("inventory-service-wsdl-url");

            return true;
        } catch (IOException ex) {
            ex.printStackTrace();
        } finally {
            if (input != null) {
                try {
                    input.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }

        return false;
    }

    /**
     * Save.
     * @return if save ok
     */
    public boolean save() {
        Properties prop = new Properties();
        OutputStream output = null;

        try {

            output = new FileOutputStream("config.properties");

            // Set the properties value
            prop.setProperty("access-service-wsdl-url", accessServiceWSDLUrl);
            prop.setProperty("usermanagement-service-wsdl-url", userManagementServiceWSDLUrl);
            prop.setProperty("order-service-wsdl-url", orderServiceWSDLUrl);
            prop.setProperty("inventory-service-wsdl-url", inventoryServiceWSDLUrl);

            // Save properties to project root folder
            prop.store(output, null);

            return true;
        } catch (IOException ex) {

            ex.printStackTrace();

            return false;
        } finally {
            if (output != null) {
                try {
                    output.close();
                } catch (IOException ex) {
                    // TODO Log
                    ex.printStackTrace();
                }
            }
        }
    }
}
