package ch.hslu.appe.gui.model;

import java.math.BigDecimal;
import java.util.List;

import javax.swing.table.AbstractTableModel;

import ch.hslu.appe.inventory.webservice.ProductDto;
import ch.hslu.appe.webservice.consumer.InventoryConsumerImpl;

/**
 * Product Table Model.
 */
@SuppressWarnings("serial")
public class ProductTableModel extends AbstractTableModel {

    private final String[] tableHeader;
    private Object[][] tableData;
    private List<ProductDto> products;

    /**
     * Constructor.
     */
    public ProductTableModel() {

        tableHeader = new String[] {"Number", "Produktname", "Preis", "Anzahl" };

        generateTableData();
    }

    /**
     * Gets the column class.
     * @param c colum
     * @return column class
     */
    @SuppressWarnings({ "unchecked", "rawtypes" })
    @Override
    public final Class getColumnClass(final int c) {
        return getValueAt(0, c).getClass();
    }

    /**
     * Gets the number of columns.
     * @return number of columns
     */
    @Override
    public final int getColumnCount() {
        return tableHeader.length;
    }

    /**
     * Gets the name of the column.
     * @param col column
     * @return name of the column
     */
    @Override
    public final String getColumnName(final int col) {
        return tableHeader[col];
    }

    /**
     * Gets the number of rows.
     * @return number of rows
     */
    @Override
    public final int getRowCount() {
        return tableData.length;
    }

    /**
     * Gets the Table Entry.
     * @param row row
     * @param col col
     * @return entry
     */
    @Override
    public final Object getValueAt(final int row, final int col) {
        return tableData[row][col];
    }

    @Override
    public final void setValueAt(final Object value, final int row, final int col) {
        tableData[row][col] = value;
        fireTableCellUpdated(row, col);
    }

    /**
     * Generate / Update Table Data.
     */
    public final void generateTableData() {

        products = InventoryConsumerImpl.getInstance().getActiveProducts();

        if (products == null) {
            return;
        }

        tableData = new Object[products.size()][4];

        for (int index = 0; index < products.size(); index++) {
            ProductDto product = products.get(index);

            tableData[index][0] = product.getNumber();
            tableData[index][1] = product.getProductname();
            float price = round(product.getPrice(), 2);
            tableData[index][2] = String.format("%.2f", price);
            tableData[index][3] = product.getAmount();
        }
    }

    /**
     * round.
     * @param number number
     * @param decimalPlace decimalPlace
     * @return formated float
     */
    private float round(final float number, final int decimalPlace) {
        BigDecimal bd = new BigDecimal(Float.toString(number));
        bd = bd.setScale(decimalPlace, BigDecimal.ROUND_HALF_UP);
        return bd.floatValue();
    }
}
